package gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollBar;
import javax.swing.JSlider;
import javax.swing.JTextField;

import logic.SequenceWithRibosomes;
import logic.Translator;

public class Codons extends JApplet {

	private static final long serialVersionUID = 1L;
	// declare everything
	private boolean simulationStarted = false;
	private boolean simulationPaused = false;

	// private final int fastForwardSteps = 100;
	private final int MAX_DELAY = 60;

	private SequenceWithRibosomes seqWithRibo = new SequenceWithRibosomes();
	private final SequenceDrawing sequenceArea = new SequenceDrawing(this);

	private JSlider stepDelay = new JSlider();
	private CodonSelector sameSenseSelector = new CodonSelector(sequenceArea);

	private Thread runningSim = new Thread();
	private JTextField currentCodonField;
	private JTextField otherCodonField;
	private JTextField currentCodonAA3Field;
	private JTextField currentCodonAA1Field;
	private JLabel lblDifferentCodon;
	private final Box horizontalBox_1 = Box.createHorizontalBox();
	private JLabel elapsedTimeLabel = new JLabel("Elapsed time: 0 ms");
	private JScrollBar offsetScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);

	private class RunSimulation implements Runnable {

		@Override
		public void run() {
			try {
				if (!simulationPaused) {
					seqWithRibo.simulationStart();
				}
				sequenceArea.drawSequence(getGraphics());
				for (;;) {
					if (seqWithRibo.simulationStep()) {
						sequenceArea.drawSequence(getGraphics());
					}
					showElapsedTime();
					Thread.sleep(stepDelay.getValue() / 3);
					if (seqWithRibo.anyClashes()) {
						return;
					}
				}
			} catch (InterruptedException e) {
				// it's OK to interrupt this thread if you want to stop it
				showElapsedTime();
			}
		}
	}

	public Codons() {
		// prepare applet
		getContentPane().setLayout(
				new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		getContentPane().add(horizontalBox_1);

		horizontalBox_1.add(sequenceArea);

		sequenceArea.setBackground(Color.GRAY);
		sequenceArea.setForeground(Color.DARK_GRAY);
		sequenceArea.setSeqWithRibo(seqWithRibo);
		offsetScrollBar.setUnitIncrement(10);
		offsetScrollBar.addAdjustmentListener(new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent arg0) {
				sequenceArea.setOffset(arg0.getValue() / 3);
			}
		});
		// offsetScrollBar.setPreferredSize(new Dimension(17, 20));
		// offsetScrollBar.setMaximum(300);

		getContentPane().add(offsetScrollBar);
		offsetScrollBar.setOrientation(JScrollBar.HORIZONTAL);

		Box horizontalBox = Box.createHorizontalBox();
		horizontalBox.setPreferredSize(new Dimension(0, 100));
		getContentPane().add(horizontalBox);

		JButton btnZoomIn = new JButton("");
		btnZoomIn.setToolTipText("Zoom in the sequence");
		btnZoomIn.setIcon(new ImageIcon(Codons.class
				.getResource("/resources/zoom_in.png")));
		btnZoomIn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sequenceArea.zoomIn();
			}
		});

		Component horizontalStrut = Box.createHorizontalStrut(20);
		horizontalStrut.setMaximumSize(new Dimension(80, 32767));
		horizontalStrut.setMinimumSize(new Dimension(80, 0));
		horizontalStrut.setPreferredSize(new Dimension(80, 0));
		horizontalStrut.setSize(new Dimension(80, 0));
		horizontalBox.add(horizontalStrut);
		horizontalBox.add(btnZoomIn);

		JButton btnZoomOut = new JButton("");
		btnZoomOut.setToolTipText("Zoom out the sequence");
		btnZoomOut.setIcon(new ImageIcon(Codons.class
				.getResource("/resources/zoom_out.png")));
		btnZoomOut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sequenceArea.zoomOut();
			}
		});
		horizontalBox.add(btnZoomOut);

		JButton btnStartSimulation = new JButton("");
		btnStartSimulation.setToolTipText("Start simulation");
		btnStartSimulation.setIcon(new ImageIcon(Codons.class
				.getResource("/resources/play_icon.png")));
		btnStartSimulation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!runningSim.isAlive()) {
					simulationStarted = true;
					runningSim = new Thread(new RunSimulation());
					runningSim.start();
				} else {
					JOptionPane.showMessageDialog(new Frame(),
							"Already running");
				}
			}
		});

		Component horizontalStrut_1 = Box.createHorizontalStrut(20);
		horizontalStrut_1.setSize(new Dimension(60, 0));
		horizontalStrut_1.setPreferredSize(new Dimension(60, 0));
		horizontalStrut_1.setMinimumSize(new Dimension(60, 0));
		horizontalStrut_1.setMaximumSize(new Dimension(60, 32767));
		horizontalBox.add(horizontalStrut_1);

		Box animationControlBox = Box.createHorizontalBox();

		animationControlBox.add(btnStartSimulation);

		JButton btnStopSimulation = new JButton("");
		btnStopSimulation.setToolTipText("Stop simulation");
		btnStopSimulation.setIcon(new ImageIcon(Codons.class
				.getResource("/resources/stop_icon.png")));
		btnStopSimulation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				seqWithRibo.setElapsedTime(0);
				showElapsedTime();
				if (simulationStarted) {
					if (runningSim.isAlive()) {
						runningSim.interrupt();
					}
					seqWithRibo.simulationClean();
					// getGraphics().clearRect(0, 0, 1000, 75); <-- this line
					// caused some flicker
					sequenceArea.drawSequence(getGraphics());
					simulationStarted = false;
					simulationPaused = false;
				} else {
					JOptionPane.showMessageDialog(new Frame(),
							"No simulation running");
				}
			}
		});

		JButton btnPauseSimulation = new JButton("");
		btnPauseSimulation.setToolTipText("Pause simulation\n");
		btnPauseSimulation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (simulationStarted) {
					simulationPaused = true;
					if (runningSim.isAlive()) {
						runningSim.interrupt();
					}
				} else {
					JOptionPane.showMessageDialog(new Frame(),
							"No simulation running");
				}
			}
		});
		btnPauseSimulation.setIcon(new ImageIcon(Codons.class
				.getResource("/resources/pause_icon.png")));
		animationControlBox.add(btnPauseSimulation);
		animationControlBox.add(btnStopSimulation);

		JButton btnStepBackwards = new JButton("");
		btnStepBackwards.setToolTipText("Step backwards");
		btnStepBackwards.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				sequenceArea.drawSequence(getGraphics());
				seqWithRibo.multipleSimulationSteps(-50
						* (1 + MAX_DELAY - stepDelay.getValue()));
				sequenceArea.drawSequence(getGraphics());
				showElapsedTime();
			}
		});
		btnStepBackwards.setIcon(new ImageIcon(Codons.class
				.getResource("/resources/backward_icon.png")));
		animationControlBox.add(btnStepBackwards);

		JButton btnStepForwards = new JButton("");
		btnStepForwards.setToolTipText("Step forwards");
		btnStepForwards.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!seqWithRibo.anyClashes()) {
					if (!simulationStarted) {
						simulationStarted = true;
						runningSim = new Thread(new RunSimulation());
						seqWithRibo.simulationStart();
						sequenceArea.drawSequence(getGraphics());
					}
					seqWithRibo
							.multipleSimulationSteps(50 * (1 + MAX_DELAY - stepDelay
									.getValue()));
					sequenceArea.drawSequence(getGraphics());
					showElapsedTime();
				}
			}
		});
		btnStepForwards.setIcon(new ImageIcon(Codons.class
				.getResource("/resources/forward_icon.png")));
		animationControlBox.add(btnStepForwards);

		Box bigAnimationControlBox = Box.createVerticalBox();
		bigAnimationControlBox.add(animationControlBox);
		bigAnimationControlBox.add(elapsedTimeLabel);
		horizontalBox.add(bigAnimationControlBox);

		Component horizontalStrut_2 = Box.createHorizontalStrut(20);
		horizontalStrut_2.setSize(new Dimension(20, 0));
		horizontalStrut_2.setMinimumSize(new Dimension(30, 0));
		horizontalBox.add(horizontalStrut_2);

		JLabel lblSlower = new JLabel("Slower");
		horizontalBox.add(lblSlower);

		stepDelay = new JSlider();
		stepDelay.setInverted(true);
		stepDelay.setValue(20);
		stepDelay.setToolTipText("Simulation speed");
		horizontalBox.add(stepDelay);
		stepDelay.setMaximum(60);
		stepDelay.setMinimum(1);

		JLabel lblStepDelay = new JLabel("Faster");
		horizontalBox.add(lblStepDelay);

		Component horizontalStrut_3 = Box.createHorizontalStrut(20);
		horizontalBox.add(horizontalStrut_3);

		Box verticalBox_1 = Box.createVerticalBox();
		horizontalBox.add(verticalBox_1);

		Box verticalBox = Box.createVerticalBox();
		horizontalBox.add(verticalBox);

		Box horizontalBox2 = Box.createHorizontalBox();
		getContentPane().add(horizontalBox2);

		JButton btnDownloadSequence = new JButton(
				"<html><center>Copy&nbsp;sequence<br/>to&nbsp;clipboard</center></html>");
		btnDownloadSequence.setSize(new Dimension(200, 50));
		btnDownloadSequence.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Toolkit toolkit = Toolkit.getDefaultToolkit();
				Clipboard clipboard = toolkit.getSystemClipboard();
				StringSelection selection = new StringSelection(seqWithRibo.getSequence());
				clipboard.setContents(selection, null);
				/*
				 * SeqenceWindow seqWindow = new SeqenceWindow();
				 * seqWindow.setSequence(seqWithRibo.getSequence());
				 * seqWindow.setVisible(true);
				 */
			}
		});
		horizontalBox2.add(btnDownloadSequence);
		btnDownloadSequence.setPreferredSize(new Dimension(80, 40));

		Component horizontalStrut_4 = Box.createHorizontalStrut(20);
		horizontalStrut_4.setSize(new Dimension(20, 0));
		horizontalStrut_4.setMaximumSize(new Dimension(1000, 32767));
		horizontalBox2.add(horizontalStrut_4);

		Box verticalBox_4 = Box.createVerticalBox();
		horizontalBox2.add(verticalBox_4);

		JLabel lblCurrentCodon = new JLabel("Current codon:");
		verticalBox_4.add(lblCurrentCodon);

		currentCodonField = new JTextField();
		currentCodonField.setBackground(Color.WHITE);
		currentCodonField.setEditable(false);
		currentCodonField.setPreferredSize(new Dimension(4, 20));
		verticalBox_4.add(currentCodonField);
		currentCodonField.setColumns(3);

		currentCodonAA3Field = new JTextField();
		currentCodonAA3Field.setBackground(Color.WHITE);
		currentCodonAA3Field.setEditable(false);
		verticalBox_4.add(currentCodonAA3Field);
		currentCodonAA3Field.setColumns(10);

		currentCodonAA1Field = new JTextField();
		currentCodonAA1Field.setBackground(Color.WHITE);
		currentCodonAA1Field.setEditable(false);
		verticalBox_4.add(currentCodonAA1Field);
		currentCodonAA1Field.setColumns(10);

		Box verticalBox_5 = Box.createVerticalBox();
		horizontalBox2.add(verticalBox_5);

		JLabel lblAlternativeCodonsWith = new JLabel(
				"Alternative synonymous codons (click to mutate)");
		verticalBox_5.add(lblAlternativeCodonsWith);
		verticalBox_5.add(sameSenseSelector);

		Box verticalBox_2 = Box.createVerticalBox();
		horizontalBox2.add(verticalBox_2);

		Box verticalBox_6 = Box.createVerticalBox();
		horizontalBox2.add(verticalBox_6);

		Component verticalStrut_2 = Box.createVerticalStrut(20);
		verticalStrut_2.setMinimumSize(new Dimension(30, 20));
		verticalStrut_2.setMaximumSize(new Dimension(3000, 2000));
		verticalBox_6.add(verticalStrut_2);

		lblDifferentCodon = new JLabel("Non-synonymous codon:");
		verticalBox_6.add(lblDifferentCodon);

		otherCodonField = new JTextField();
		verticalBox_6.add(otherCodonField);
		otherCodonField.setColumns(3);

		JButton btnOtherCodonl = new JButton("Mutate");
		btnOtherCodonl.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// validate input
				String newCodon = otherCodonField.getText().toUpperCase()
						.split(" ")[0];
				if (Translator.getInstance().checkValid(newCodon)) {
					sequenceArea.mutate(newCodon);
					sameSenseSelector.showAlternatives(Translator.getInstance()
							.alternatives(newCodon));
				} else
					JOptionPane.showMessageDialog(new Frame(),
							"Incorrect codon!");
			}
		});
		verticalBox_6.add(btnOtherCodonl);

		Component verticalStrut_3 = Box.createVerticalStrut(20);
		verticalStrut_3.setPreferredSize(new Dimension(0, 10));
		verticalStrut_3.setMinimumSize(new Dimension(10, 10));
		verticalStrut_3.setMaximumSize(new Dimension(3000, 3000));
		verticalBox_6.add(verticalStrut_3);

	}

	private void mutateToAny() {
		lblDifferentCodon.setText("Non-synonymous codon:");
		otherCodonField.setBackground(Color.WHITE);
		otherCodonField.setText("");
		otherCodonField.setEditable(true);
	}

	private void mutateBack(String codon) {
		lblDifferentCodon.setText("Original codon:");
		otherCodonField.setBackground(Color.LIGHT_GRAY);
		otherCodonField.setText(codon + "    " + seqWithRibo.getDelay(codon));
		otherCodonField.setEditable(false);
	}

	public void setCurrentCodon(int index) {
		if (sequenceArea.getZoomLevel() < 0)
			return;
		if (index >= (seqWithRibo.getCodons().size() - 1))
			return;
		String codonText = seqWithRibo.toString().substring(index * 3,
				index * 3 + 3);
		currentCodonField.setText(codonText + "    "
				+ seqWithRibo.getDelay(codonText));
		sameSenseSelector.showAlternatives(Translator.getInstance()
				.alternatives(codonText));
		currentCodonAA1Field.setText(Translator.getInstance().translateCodon(
				codonText));
		currentCodonAA3Field.setText(Translator.getInstance().oneToThree(
				currentCodonAA1Field.getText().charAt(0)));
		// maybe we want to revert mutation
		if (seqWithRibo.getOriginalCodons().containsKey(index)) {
			mutateBack(seqWithRibo.getOriginalCodons().get(index));
		} else {
			mutateToAny();
		}
	}

	@Override
	public void init() {
		super.start();
		seqWithRibo.setDataFromParams(getParameter("seq_str"),
				getParameter("next_ribo_delay"), getParameter("delays"),
				getParameter("human").equals("human_manage"));
		try {
			offsetScrollBar.setMaximum(getParameter("seq_str").length());
		} catch (IllegalArgumentException e) {
		}
	}

	@Override
	public void start() {
		super.start();
		// sequenceArea.drawSequence(getGraphics());
	}

	private void showElapsedTime() {
		elapsedTimeLabel.setText("Elapsed time: "
				+ Integer.toString(seqWithRibo.getElapsedTime()) + " ms");
	};
}
