package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import logic.SequenceWithRibosomes;

class SequenceDrawing extends JPanel {

	private static final long serialVersionUID = 1L;
	private SequenceWithRibosomes seqWithRibo = new SequenceWithRibosomes();
	private Codons parent;
	// BufferStrategy strategy = get

	private int zoomLevel = 10;
	private final int zoomStep = 2;
	private final int zoomMin = -20;
	private final int zoomMax = 14;
	private final int RIBOSOME_RADIUS = 5;

	private int currentCodonIndex = -1;
	private int drawingOffset = 0;

	private final Color RIBOSOME_COLOR = new Color(20, 80, 255, 150);

	// private final double RIBOSOME_POSITION_MULT = 2.37;
	private int codonPositionMult = 32;

	public SequenceDrawing(Codons parentApplet) {
		setDoubleBuffered(true);
		parent = parentApplet;
		addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				int kodon_num = (int) (e.getX() / codonPositionMult)
						+ drawingOffset;
				if (kodon_num <= seqWithRibo.getCodons().size() - 2)
					setCurrentCodon(kodon_num);
			}
		});
		// set a preferred size for the custom panel.
		setPreferredSize(new Dimension(1000, 260));
		addMouseWheelListener(new MouseWheelListener() {
			@Override
			public void mouseWheelMoved(MouseWheelEvent e) {
				if (e.getWheelRotation() < 0)
					zoomIn();
				else
					zoomOut();
			}
		});
	}

	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		drawSequence(g);
	}

	public void zoomOut() {
		if (zoomLevel < 0){
			if (seqWithRibo.getSequence().length() / (-zoomLevel / 2 + 1) < 900) 
				return;
			}
		if (zoomLevel > zoomMin) {
			zoomLevel -= zoomStep;
			drawSequence(getGraphics());
		}
	}

	public void zoomIn() {
		if (zoomLevel < zoomMax) {
			zoomLevel += zoomStep;
			drawSequence(getGraphics());
		}
	}

	public void setSeqWithRibo(SequenceWithRibosomes seq) {
		seqWithRibo = seq;
	}

	public SequenceWithRibosomes getSeqWithRibo() {
		return seqWithRibo;
	}

	void drawSequence(Graphics graph) {
		if (zoomLevel <= 0) {
			drawSmallSequence(graph);
			return;
		}
		Image bufferedImage = createImage(getWidth(), getHeight());
		Graphics buffer = bufferedImage.getGraphics();
		// add spaces to string (split codons)
		StringBuilder spacedSeqBuilder = new StringBuilder();
		String spacedSeq, spacedProtSeq;
		for (String codon : seqWithRibo.getCodons()) {
			spacedSeqBuilder.append(codon);
			spacedSeqBuilder.append(" ");
		}
		spacedSeq = spacedSeqBuilder.toString();
		spacedSeqBuilder = new StringBuilder();
		String proteinSequence = seqWithRibo.getProteinSequence();
		for (int i = 0; i < proteinSequence.length(); i++) {
			spacedSeqBuilder.append(" ");
			spacedSeqBuilder.append(proteinSequence.charAt(i));
			spacedSeqBuilder.append("  ");
		}
		spacedProtSeq = spacedSeqBuilder.toString();

		// check zoom level and act accordingly
		buffer.setColor(Color.LIGHT_GRAY);
		buffer.fillRect(0, 0, 1000, 85);
		buffer.setColor(Color.BLACK);
		Font font = new Font("Monospaced", Font.PLAIN, zoomLevel);
		buffer.setFont(font);
		int realOffset = buffer.getFontMetrics().stringWidth("AAA ")
				* drawingOffset;

		// write ribosomes with right zoom
		boolean[] ribosomes = seqWithRibo.getRibosomes();
		boolean[] clashes = seqWithRibo.getClashes();
		int pos;
		buffer.setColor(RIBOSOME_COLOR);
		for (int i = 0; i < ribosomes.length; i++) {
			pos = (int) ((i + 0.25) * codonPositionMult);
			if (clashes[i]) {
				buffer.setColor(Color.RED);
				buffer.fillRect(pos - (RIBOSOME_RADIUS + 2)
						* buffer.getFontMetrics().stringWidth("AAA")
						- realOffset, 40, (2 * RIBOSOME_RADIUS)
						* buffer.getFontMetrics().stringWidth("AAA ")
						+ buffer.getFontMetrics().stringWidth("AAA"), buffer
						.getFontMetrics().getHeight());
				/*
				 * buffer.fillRect(pos - realOffset - (RIBOSOME_RADIUS + 2), 40,
				 * (2 * RIBOSOME_RADIUS)
				 * buffer.getFontMetrics().stringWidth("AAA "),
				 * buffer.getFontMetrics().getHeight());
				 */
				buffer.setFont(new Font("Monospaced", Font.BOLD, 30));
				buffer.drawString("!", pos - realOffset, 35);
				buffer.setFont(font);
			} else if (ribosomes[i]) {
				buffer.fillOval(pos - realOffset, 20 - (zoomMax - zoomLevel),
						zoomLevel, (int) (15 + (zoomMax - zoomLevel)));
				buffer.fillRect(pos - (RIBOSOME_RADIUS + 2)
						* buffer.getFontMetrics().stringWidth("AAA")
						- realOffset, 40, (2 * RIBOSOME_RADIUS)
						* buffer.getFontMetrics().stringWidth("AAA ")
						+ buffer.getFontMetrics().stringWidth("AAA"), buffer
						.getFontMetrics().getHeight());
			}
		}

		// write sequence
		buffer.setFont(font);
		buffer.setColor(Color.BLACK);
		buffer.drawString(spacedSeq, -realOffset, 40 + zoomLevel);
		buffer.drawString(spacedProtSeq, -realOffset, 40 + zoomLevel * 2);

		codonPositionMult = buffer.getFontMetrics().stringWidth("AUG ");
		if (currentCodonIndex != -1) {
			buffer.drawLine((int) (currentCodonIndex * codonPositionMult)
					- realOffset, 42 + zoomLevel,
					(int) ((currentCodonIndex + 0.75) * codonPositionMult)
							- realOffset, 42 + zoomLevel);
		}
		// put mutants in frames
		buffer.setColor(Color.RED);
		for (int i : seqWithRibo.getOriginalCodons().keySet()) {
			buffer.drawRect(i * buffer.getFontMetrics().stringWidth("AAA ")
					- realOffset, 40, buffer.getFontMetrics()
					.stringWidth("AAA"), zoomLevel * 2);
		}

		// finishing touches
		buffer.setColor(Color.BLACK);
		buffer.drawLine(-realOffset, 38,
				buffer.getFontMetrics().stringWidth(spacedSeq) - realOffset, 38);

		int charSize = buffer.getFontMetrics().stringWidth(" ");
		int rulerScale = (zoomLevel > 8) ? 5 : ((zoomLevel > 4) ? 10 : 50);
		buffer.setFont(new Font("Monospaced", Font.PLAIN, 10));
		for (int i = rulerScale; i <= seqWithRibo.length() / 3; i += rulerScale) {
			int x = -realOffset + (charSize * 4 * (i - 1));
			if (zoomLevel <= 12)
				buffer.drawString(Integer.toString(i), x, 70);
			else
				buffer.drawString(Integer.toString(i), x, 80);
		}
		graph.drawImage(bufferedImage, 0, 0, this);
	}

	private void drawSmallSequence(Graphics graph) {
		int realZoom = -zoomLevel / 2 + 1;
		Image bufferedImage = createImage(getWidth(), getHeight());
		Graphics buffer = bufferedImage.getGraphics();
		int realOffset = buffer.getFontMetrics().stringWidth("-")
				* drawingOffset / realZoom;
		// add spaces to string (split codons)
		StringBuilder dotSeqBuilder = new StringBuilder();
		String dotSeq;
		dotSeqBuilder = new StringBuilder();
		String proteinSequence = seqWithRibo.getProteinSequence();
		for (int i = 0; i < proteinSequence.length() / realZoom; i++) {
			dotSeqBuilder.append("-");
		}
		dotSeq = dotSeqBuilder.toString();

		buffer.setColor(Color.LIGHT_GRAY);
		buffer.fillRect(0, 0, 1000, 85);
		buffer.setColor(Color.BLACK);

		buffer.drawString(dotSeq, -realOffset, 46);

		// draw ribosomes
		buffer.setColor(RIBOSOME_COLOR);
		boolean[] ribosomes = seqWithRibo.getRibosomes();
		boolean[] clashes = seqWithRibo.getClashes();
		for (int i = 0; i < ribosomes.length; i++) {
			int pos = (int) (i * 4 / realZoom);
			if (clashes[i]) {
				buffer.setColor(Color.RED);
				buffer.drawString("!", pos - realOffset, 35);
			} else if (ribosomes[i]) {
				buffer.fillOval(pos - realOffset, 25, 10, 10);
			}
		}
		// mark mutants
		buffer.setColor(Color.RED);
		for (int i : seqWithRibo.getOriginalCodons().keySet()) {
			buffer.drawString("-", -realOffset + i / realZoom, 46);
		}

		// finishing touches
		buffer.setColor(Color.BLACK);
		buffer.drawLine(0, 38, 1000, 38);

		graph.drawImage(bufferedImage, 0, 0, this);
	}

	private void setCurrentCodon(int index) {
		currentCodonIndex = index;
		parent.setCurrentCodon(index);
		drawSequence(getGraphics());
	}

	void mutate(String newCodon) {
		if (currentCodonIndex < 0) {
			JOptionPane.showMessageDialog(new Frame(), "No codon selected!");
			return;
		}
		seqWithRibo.mutate(currentCodonIndex, newCodon.replace("T", "U"));
		parent.setCurrentCodon(currentCodonIndex);
		drawSequence(getGraphics());
	}

	void setOffset(int newOffset) {
		drawingOffset = (newOffset > 0) ? newOffset : 0;
		drawSequence(getGraphics());
	}

	public int getZoomLevel() {
		return zoomLevel;
	}
}