package logic;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.regex.Pattern;

public class Translator {
	// singleton
	private static Translator instance = new Translator();
	
	private Hashtable<String, Character> oneLetterCode = new Hashtable<String, Character>();
	private Hashtable<Character, String> threeLetterCode = new Hashtable<Character, String>();
	private Pattern valid = Pattern.compile("[ATUCG][ATUCG][ATUCG]");

	private Translator() {
		InputStream stream = getClass().getResourceAsStream(
				"/resources/kod_jednoliterowy.tsv");
		InputStreamReader streamReader = new InputStreamReader(stream);
		BufferedReader bufferedReader = new BufferedReader(streamReader);
		String line;
		try {
			line = bufferedReader.readLine();
		} catch (IOException e) {
			line = null;
		}
		while (line != null) {
			String[] kv = line.split("\\t");
			oneLetterCode.put(kv[0], kv[1].charAt(0));
			try {
				line = bufferedReader.readLine();
			} catch (IOException e) {
				line = null;
			}
		}
		// Translate 1 to 3 letters
		stream = getClass().getResourceAsStream(
				"/resources/kod_trzyliterowy.tsv");
		streamReader = new InputStreamReader(stream);
		bufferedReader = new BufferedReader(streamReader);
		line = null;
		try {
			line = bufferedReader.readLine();
		} catch (IOException e) {
			line = null;
		}
		while (line != null) {
			String[] kv = line.split("\\t");
			threeLetterCode.put(kv[0].charAt(0), kv[1].toUpperCase());
			try {
				line = bufferedReader.readLine();
			} catch (IOException e) {
				line = null;
			}
		}
	}
	
	public static Translator getInstance() {
        return instance;
    }
	
	public void makeHuman(){
		oneLetterCode.put("UGA", 'U');
		threeLetterCode.put('U', "Sel");
	}

	public String translateCodon(String codon) {
		String trueCodon = codon.replace("T", "U");
		try{
			return Character.toString(oneLetterCode.get(trueCodon));
		} catch (NullPointerException e) {
			return "";
		}
	}
	
	public String oneToThree(char aa1){
		return threeLetterCode.get(aa1);
	}
	
	public ArrayList<String> reverseAminoacid(String aa){
		ArrayList<String> ans = new ArrayList<String>();
		Enumeration<String> keys = oneLetterCode.keys();
		String key, value;
		while (keys.hasMoreElements()){
			key = keys.nextElement();
			value = Character.toString(oneLetterCode.get(key));
			if (value.equals(aa)){
				ans.add(key.replace("T", "U"));
			}
		}
		return ans;
	}
	
	public ArrayList<String> sameSense(String codon){
		return reverseAminoacid(translateCodon(codon));
	}
	
	public ArrayList<String> alternatives(String codon){
		ArrayList<String> ans = reverseAminoacid(translateCodon(codon));
		ans.remove(codon.replace("T", "U"));
		ans.remove(codon);
		return ans;
	}
	
	public boolean checkValid(String codon){
		if (!valid.matcher(codon).matches())
			return false;
		return translateCodon(codon).length()>0;
	}
}
